/**
 * Licensed to Apereo under one or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information regarding copyright ownership. Apereo
 * licenses this file to you under the Apache License, Version 2.0 (the "License"); you may not use
 * this file except in compliance with the License. You may obtain a copy of the License at the
 * following location:
 *
 * <p>http://www.apache.org/licenses/LICENSE-2.0
 *
 * <p>Unless required by applicable law or agreed to in writing, software distributed under the
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apereo.portal.rendering;

import javax.servlet.http.HttpServletRequest;
import org.apereo.portal.portlet.om.IPortletDefinition;
import org.apereo.portal.portlet.om.IPortletEntity;
import org.apereo.portal.portlet.om.IPortletWindow;
import org.apereo.portal.portlet.om.IPortletWindowId;
import org.apereo.portal.portlet.registry.IPortletWindowRegistry;
import org.apereo.portal.url.IPortalRequestInfo;
import org.apereo.portal.url.IUrlSyntaxProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class RequestRenderingPipelineUtilsImpl implements RequestRenderingPipelineUtils {

    private IPortletWindowRegistry portletWindowRegistry;
    private IUrlSyntaxProvider urlSyntaxProvider;

    @Autowired
    public void setUrlSyntaxProvider(IUrlSyntaxProvider provider) {
        this.urlSyntaxProvider = provider;
    }

    @Autowired
    public void setPortletWindowRegistry(IPortletWindowRegistry portletWindowRegistry) {
        this.portletWindowRegistry = portletWindowRegistry;
    }

    @Override
    public IPortletDefinition getPortletDefinitionFromServletRequest(HttpServletRequest request) {
        final IPortalRequestInfo portalRequestInfo =
                this.urlSyntaxProvider.getPortalRequestInfo(request);
        if (portalRequestInfo != null && portalRequestInfo.getTargetedPortletWindowId() != null) {
            IPortletWindowId targetedPortletWindowId =
                    portalRequestInfo.getTargetedPortletWindowId();
            IPortletWindow portletWindow =
                    this.portletWindowRegistry.getPortletWindow(request, targetedPortletWindowId);
            if (portletWindow != null && portletWindow.getPortletEntity() != null) {
                final IPortletEntity portletEntity = portletWindow.getPortletEntity();
                IPortletDefinition portletDefinition = portletEntity.getPortletDefinition();
                return portletDefinition;
            }
        }

        return null;
    }
}
