package com.github.privacystreams.core.actions.callback;

import com.github.privacystreams.core.Function;
import com.github.privacystreams.core.Item;
import com.github.privacystreams.core.actions.MStreamAction;
import com.github.privacystreams.core.actions.SStreamAction;
import com.github.privacystreams.utils.Assertions;

/**
 * Callback with a field value in a item once the field value is present
 */

class IfFieldPresentCallback2<TValue, Void> extends SStreamAction {

    private final String fieldToSelect;
    private final Function<TValue, Void> fieldValueCallback;

    IfFieldPresentCallback2(String fieldToSelect, Function<TValue, Void> fieldValueCallback) {
        this.fieldToSelect = Assertions.notNull("fieldToSelect", fieldToSelect);
        this.fieldValueCallback = Assertions.notNull("fieldValueCallback", fieldValueCallback);
        this.addParameters(fieldToSelect, fieldValueCallback);
    }

    @Override
    protected void onInput(Item item) {
        if (item.isEndOfStream()) {
            this.finish();
            return;
        }
        TValue fieldValue = item.getValueByField(this.fieldToSelect);
        if (fieldValue == null) return;
        this.fieldValueCallback.apply(this.getUQI(), fieldValue);
    }

}
