/**
 *                                  Apache License
 *                            Version 2.0, January 2004
 *                         http://www.apache.org/licenses/
 *
 *    TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION
 *
 *    1. Definitions.
 *
 *       "License" shall mean the terms and conditions for use, reproduction,
 *       and distribution as defined by Sections 1 through 9 of this document.
 *
 *       "Licensor" shall mean the copyright owner or entity authorized by
 *       the copyright owner that is granting the License.
 *
 *       "Legal Entity" shall mean the union of the acting entity and all
 *       other entities that control, are controlled by, or are under common
 *       control with that entity. For the purposes of this definition,
 *       "control" means (i) the power, direct or indirect, to cause the
 *       direction or management of such entity, whether by contract or
 *       otherwise, or (ii) ownership of fifty percent (50%) or more of the
 *       outstanding shares, or (iii) beneficial ownership of such entity.
 *
 *       "You" (or "Your") shall mean an individual or Legal Entity
 *       exercising permissions granted by this License.
 *
 *       "Source" form shall mean the preferred form for making modifications,
 *       including but not limited to software source code, documentation
 *       source, and configuration files.
 *
 *       "Object" form shall mean any form resulting from mechanical
 *       transformation or translation of a Source form, including but
 *       not limited to compiled object code, generated documentation,
 *       and conversions to other media types.
 *
 *       "Work" shall mean the work of authorship, whether in Source or
 *       Object form, made available under the License, as indicated by a
 *       copyright notice that is included in or attached to the work
 *       (an example is provided in the Appendix below).
 *
 *       "Derivative Works" shall mean any work, whether in Source or Object
 *       form, that is based on (or derived from) the Work and for which the
 *       editorial revisions, annotations, elaborations, or other modifications
 *       represent, as a whole, an original work of authorship. For the purposes
 *       of this License, Derivative Works shall not include works that remain
 *       separable from, or merely link (or bind by name) to the interfaces of,
 *       the Work and Derivative Works thereof.
 *
 *       "Contribution" shall mean any work of authorship, including
 *       the original version of the Work and any modifications or additions
 *       to that Work or Derivative Works thereof, that is intentionally
 *       submitted to Licensor for inclusion in the Work by the copyright owner
 *       or by an individual or Legal Entity authorized to submit on behalf of
 *       the copyright owner. For the purposes of this definition, "submitted"
 *       means any form of electronic, verbal, or written communication sent
 *       to the Licensor or its representatives, including but not limited to
 *       communication on electronic mailing lists, source code control systems,
 *       and issue tracking systems that are managed by, or on behalf of, the
 *       Licensor for the purpose of discussing and improving the Work, but
 *       excluding communication that is conspicuously marked or otherwise
 *       designated in writing by the copyright owner as "Not a Contribution."
 *
 *       "Contributor" shall mean Licensor and any individual or Legal Entity
 *       on behalf of whom a Contribution has been received by Licensor and
 *       subsequently incorporated within the Work.
 *
 *    2. Grant of Copyright License. Subject to the terms and conditions of
 *       this License, each Contributor hereby grants to You a perpetual,
 *       worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *       copyright license to reproduce, prepare Derivative Works of,
 *       publicly display, publicly perform, sublicense, and distribute the
 *       Work and such Derivative Works in Source or Object form.
 *
 *    3. Grant of Patent License. Subject to the terms and conditions of
 *       this License, each Contributor hereby grants to You a perpetual,
 *       worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *       (except as stated in this section) patent license to make, have made,
 *       use, offer to sell, sell, import, and otherwise transfer the Work,
 *       where such license applies only to those patent claims licensable
 *       by such Contributor that are necessarily infringed by their
 *       Contribution(s) alone or by combination of their Contribution(s)
 *       with the Work to which such Contribution(s) was submitted. If You
 *       institute patent litigation against any entity (including a
 *       cross-claim or counterclaim in a lawsuit) alleging that the Work
 *       or a Contribution incorporated within the Work constitutes direct
 *       or contributory patent infringement, then any patent licenses
 *       granted to You under this License for that Work shall terminate
 *       as of the date such litigation is filed.
 *
 *    4. Redistribution. You may reproduce and distribute copies of the
 *       Work or Derivative Works thereof in any medium, with or without
 *       modifications, and in Source or Object form, provided that You
 *       meet the following conditions:
 *
 *       (a) You must give any other recipients of the Work or
 *           Derivative Works a copy of this License; and
 *
 *       (b) You must cause any modified files to carry prominent notices
 *           stating that You changed the files; and
 *
 *       (c) You must retain, in the Source form of any Derivative Works
 *           that You distribute, all copyright, patent, trademark, and
 *           attribution notices from the Source form of the Work,
 *           excluding those notices that do not pertain to any part of
 *           the Derivative Works; and
 *
 *       (d) If the Work includes a "NOTICE" text file as part of its
 *           distribution, then any Derivative Works that You distribute must
 *           include a readable copy of the attribution notices contained
 *           within such NOTICE file, excluding those notices that do not
 *           pertain to any part of the Derivative Works, in at least one
 *           of the following places: within a NOTICE text file distributed
 *           as part of the Derivative Works; within the Source form or
 *           documentation, if provided along with the Derivative Works; or,
 *           within a display generated by the Derivative Works, if and
 *           wherever such third-party notices normally appear. The contents
 *           of the NOTICE file are for informational purposes only and
 *           do not modify the License. You may add Your own attribution
 *           notices within Derivative Works that You distribute, alongside
 *           or as an addendum to the NOTICE text from the Work, provided
 *           that such additional attribution notices cannot be construed
 *           as modifying the License.
 *
 *       You may add Your own copyright statement to Your modifications and
 *       may provide additional or different license terms and conditions
 *       for use, reproduction, or distribution of Your modifications, or
 *       for any such Derivative Works as a whole, provided Your use,
 *       reproduction, and distribution of the Work otherwise complies with
 *       the conditions stated in this License.
 *
 *    5. Submission of Contributions. Unless You explicitly state otherwise,
 *       any Contribution intentionally submitted for inclusion in the Work
 *       by You to the Licensor shall be under the terms and conditions of
 *       this License, without any additional terms or conditions.
 *       Notwithstanding the above, nothing herein shall supersede or modify
 *       the terms of any separate license agreement you may have executed
 *       with Licensor regarding such Contributions.
 *
 *    6. Trademarks. This License does not grant permission to use the trade
 *       names, trademarks, service marks, or product names of the Licensor,
 *       except as required for reasonable and customary use in describing the
 *       origin of the Work and reproducing the content of the NOTICE file.
 *
 *    7. Disclaimer of Warranty. Unless required by applicable law or
 *       agreed to in writing, Licensor provides the Work (and each
 *       Contributor provides its Contributions) on an "AS IS" BASIS,
 *       WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 *       implied, including, without limitation, any warranties or conditions
 *       of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
 *       PARTICULAR PURPOSE. You are solely responsible for determining the
 *       appropriateness of using or redistributing the Work and assume any
 *       risks associated with Your exercise of permissions under this License.
 *
 *    8. Limitation of Liability. In no event and under no legal theory,
 *       whether in tort (including negligence), contract, or otherwise,
 *       unless required by applicable law (such as deliberate and grossly
 *       negligent acts) or agreed to in writing, shall any Contributor be
 *       liable to You for damages, including any direct, indirect, special,
 *       incidental, or consequential damages of any character arising as a
 *       result of this License or out of the use or inability to use the
 *       Work (including but not limited to damages for loss of goodwill,
 *       work stoppage, computer failure or malfunction, or any and all
 *       other commercial damages or losses), even if such Contributor
 *       has been advised of the possibility of such damages.
 *
 *    9. Accepting Warranty or Additional Liability. While redistributing
 *       the Work or Derivative Works thereof, You may choose to offer,
 *       and charge a fee for, acceptance of support, warranty, indemnity,
 *       or other liability obligations and/or rights consistent with this
 *       License. However, in accepting such obligations, You may act only
 *       on Your own behalf and on Your sole responsibility, not on behalf
 *       of any other Contributor, and only if You agree to indemnify,
 *       defend, and hold each Contributor harmless for any liability
 *       incurred by, or claims asserted against, such Contributor by reason
 *       of your accepting any such warranty or additional liability.
 *
 *    END OF TERMS AND CONDITIONS
 *
 *    APPENDIX: How to apply the Apache License to your work.
 *
 *       To apply the Apache License to your work, attach the following
 *       boilerplate notice, with the fields enclosed by brackets "{}"
 *       replaced with your own identifying information. (Don't include
 *       the brackets!)  The text should be enclosed in the appropriate
 *       comment syntax for the file format. We also recommend that a
 *       file or class name and description of purpose be included on the
 *       same "printed page" as the copyright notice for easier
 *       identification within third-party archives.
 *
 *    Copyright 2014 Edgar Espina
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package org.jooby.spec;

import static java.util.Objects.requireNonNull;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.lang.reflect.Type;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Optional;
import java.util.Set;

import org.jooby.Env;
import org.jooby.Jooby;
import org.jooby.Request;
import org.jooby.Response;
import org.jooby.Route;
import org.jooby.Session;
import org.jooby.internal.RouteMetadata;
import org.jooby.internal.mvc.RequestParamNameProviderImpl;
import org.jooby.internal.spec.AppCollector;
import org.jooby.internal.spec.Context;
import org.jooby.internal.spec.ContextImpl;
import org.jooby.internal.spec.DocCollector;
import org.jooby.internal.spec.ResponseTypeCollector;
import org.jooby.internal.spec.RouteCollector;
import org.jooby.internal.spec.RouteParamCollector;
import org.jooby.internal.spec.RouteParamImpl;
import org.jooby.internal.spec.RouteResponseImpl;
import org.jooby.internal.spec.RouteSpecImpl;
import org.jooby.internal.spec.SourceResolver;
import org.jooby.internal.spec.SourceResolverImpl;
import org.jooby.internal.spec.TypeFromDoc;
import org.jooby.internal.spec.TypeResolverImpl;
import org.jooby.mvc.Body;
import org.jooby.mvc.Flash;
import org.jooby.mvc.Header;
import org.jooby.mvc.Local;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.github.javaparser.JavaParser;
import com.github.javaparser.ast.CompilationUnit;
import com.github.javaparser.ast.Node;
import com.google.common.base.Throwables;
import com.google.common.collect.Sets;
import com.typesafe.config.ConfigFactory;

/**
 * <p>
 * Process and collect {@link RouteSpec} from {@link Jooby} app.
 * </p>
 *
 * @author edgar
 * @since 0.15.0
 */
public class RouteProcessor {

  /** SKIP MVC parameters of type: */
  @SuppressWarnings("rawtypes")
  private static final Set<Class> SKIP = Sets.newHashSet(Request.class, Response.class,
      Session.class, Route.class, Route.Chain.class);

  /** The logging system. */
  private final Logger log = LoggerFactory.getLogger(getClass());

  /**
   * Process a {@link Jooby} application and collect {@link RouteSpec}.
   *
   * @param app A jooby app to process.
   * @return List of route specs.
   */
  public List<RouteSpec> process(final Jooby app) {
    requireNonNull(app, "App is required.");
    return process(app, new File(System.getProperty("user.dir")).toPath());
  }

  /**
   * Process a {@link Jooby} application and collect {@link RouteSpec}.
   *
   * @param app A jooby app to process.
   * @param srcdir Basedir where source code is located. Useful for extracting doc.
   * @return List of route specs.
   */
  public List<RouteSpec> process(final Jooby app, final Path srcdir) {
    return processInternal(app, srcdir, null);
  }

  /**
   * Process a {@link Jooby} application and collect {@link RouteSpec}, but also save a compiled
   * version in the given outdir.
   *
   * @param app A jooby app to process.
   * @param srcdir Basedir where source code is located. Useful for extracting doc.
   * @param outdir Where to save the compiled route specs.
   * @return List of route specs.
   */
  public List<RouteSpec> compile(final Jooby app, final Path srcdir, final Path outdir) {
    requireNonNull(app, "App is required.");
    requireNonNull(srcdir, "Source dir is required.");
    requireNonNull(outdir, "Out dir is required.");

    return processInternal(app, srcdir, outdir);
  }

  /**
   * Process a {@link Jooby} application and collect {@link RouteSpec}.
   *
   * @param appClass A jooby class to process.
   * @param routes Routes to process.
   * @return List of route specs.
   */
  public List<RouteSpec> process(final Class<? extends Jooby> appClass,
      final List<Route.Definition> routes) {
    requireNonNull(appClass, "App class is required.");
    requireNonNull(routes, "Routes are required.");
    return processInternal(appClass, routes, new File(System.getProperty("user.dir")).toPath(),
        null);
  }

  /**
   * Process a {@link Jooby} application and collect {@link RouteSpec}.
   *
   * @param appClass A jooby class to process.
   * @param routes Routes to process.
   * @param srcdir Basedir where source code is located. Useful for extracting doc.
   * @return List of route specs.
   */
  public List<RouteSpec> process(final Class<? extends Jooby> appClass,
      final List<Route.Definition> routes, final Path srcdir) {
    requireNonNull(appClass, "App class is required.");
    requireNonNull(routes, "Routes are required.");
    requireNonNull(srcdir, "Source dir is required.");
    return processInternal(appClass, routes, srcdir, null);
  }

  /**
   * Process a {@link Jooby} application and collect {@link RouteSpec}.
   *
   * @param appClass A jooby app to process.
   * @param routes Routes to process.
   * @param srcdir Basedir where source code is located. Useful for extracting doc.
   * @param outdir Strategy where to save the compiled route specs.
   * @return List of route specs.
   */
  public List<RouteSpec> compile(final Class<? extends Jooby> appClass,
      final List<Route.Definition> routes, final Path srcdir, final Path outdir) {
    requireNonNull(appClass, "App class is required.");
    requireNonNull(routes, "Routes are required.");
    requireNonNull(srcdir, "Source dir is required.");
    requireNonNull(outdir, "Out dir is required.");
    return processInternal(appClass, routes, srcdir, outdir);
  }

  /**
   * Process a {@link Jooby} application and collect {@link RouteSpec}.
   *
   * @param app A jooby app to process.
   * @param srcdir Basedir where source code is located. Useful for extracting doc.
   * @param outdir Where to save the compiled route specs.
   * @return List of route specs.
   */
  private List<RouteSpec> processInternal(final Jooby app, final Path srcdir, final Path outdir) {
    List<Route.Definition> routes = Jooby.exportRoutes(app);
    return processInternal(app.getClass(), routes, srcdir, outdir);
  }

  /**
   * Process a {@link Jooby} application and collect {@link RouteSpec}.
   *
   * @param app A jooby app to process.
   * @param srcdir Basedir where source code is located. Useful for extracting doc.
   * @param outdir Strategy where to save the compiled route specs.
   * @return List of route specs.
   */
  @SuppressWarnings("unchecked")
  private List<RouteSpec> processInternal(final Class<? extends Jooby> appClass,
      final List<Route.Definition> routes, final Path srcdir, final Path outdir) {
    log.debug("processing {}.spec", appClass.getName());
    List<RouteSpec> specs = new ArrayList<>();
    try {
      /**
       * Source resolver.
       */
      SourceResolver src = new SourceResolverImpl(srcdir);

      /**
       * Context with type resolver.
       */
      Context ctx = new ContextImpl(new TypeResolverImpl(appClass.getClassLoader()), src);

      Optional<List<RouteSpec>> ifspecs = ctx.parseSpec(appClass);
      if (ifspecs.isPresent()) {
        // compiled version found.
        return ifspecs.get();
      }

      /** Collect all routes and process them. */
      Set<String> owners = new HashSet<>();
      owners.add(appClass.getName());

      List<Entry<Object, Node>> routeNodes = new ArrayList<>();
      try {
        /**
         * Main AST
         */
        CompilationUnit unit = JavaParser.parse(src.resolveSource(appClass).get(), true);

        /**
         * Find out app node.
         */
        Node appNode = new AppCollector().accept(unit, ctx);
        if (appNode == null) {
          throw new IllegalStateException("Default constructor not found");
        }

        routeNodes = new RouteCollector(owners::add).accept(appNode, ctx);
      } catch (Exception x) {
        // ignore source code error
        log.debug("source code not found", x);
      }

      int j = 0;
      for (int i = 0; i < routes.size(); i++) {
        Route.Definition route = routes.get(i);
        Object cursor = null;
        Method method = null;
        try {
          Route.Filter handler = route.filter();
          // find out where the route was defined
          final String owner;
          if (handler instanceof Route.MethodHandler) {
            method = ((Route.MethodHandler) handler).method();
            owner = method.getDeclaringClass().getName();
            owners.add(owner);
          } else {
            // lambda script
            owner = handler.getClass().getName();
          }

          if (owners.stream().filter(o -> owner.startsWith(o)).findFirst().isPresent()) {

            log.debug("    found {} {}", route.method(), route.pattern());
            Entry<Object, Node> entry = routeNodes.get(j++);

            Object candidate = entry.getKey();
            if (candidate instanceof Node) {
              cursor = candidate;
              log.debug("\n{}\n", candidate);
              /** doc and response codes . */
              Map<String, Object> doc = new DocCollector(log).accept((Node) candidate,
                  route.method(), ctx);
              Map<Integer, String> codes = (Map<Integer, String>) doc.remove("@statusCodes");
              String desc = (String) doc.remove("@text");
              String summary = (String) doc.remove("@summary");
              String retDoc = (String) doc.remove("@return");
              Type retType = (Type) doc.remove("@type");

              /** params and return type */
              List<RouteParam> params;
              RouteResponse rsp;
              String name = route.name();
              if (method == null) {
                // script params
                params = new RouteParamCollector(doc, route.method(), route.pattern())
                    .accept(entry.getValue(), ctx);
                // script response
                rsp = new ResponseTypeCollector().accept(entry.getValue(), ctx, retType, retDoc,
                    codes);
              } else {
                name = method.getName();
                params = mvcParams(route, method, doc);
                rsp = new RouteResponseImpl(
                    retType == null ? method.getGenericReturnType() : retType,
                    retDoc, codes);
              }

              // test if body param is present, when present all the other params are set to query
              params.stream()
                  .filter(p -> p.paramType() == RouteParamType.BODY)
                  .findFirst()
                  .ifPresent(p -> {
                    params.stream().filter(it -> it != p)
                        .forEach(it -> it.paramType(RouteParamType.QUERY));
                  });
              // ovewrite param type if need it
              params.stream()
                  .forEach(p -> {
                    p.doc().ifPresent(pdoc -> {
                      TypeFromDoc.parse(entry.getValue(), ctx, pdoc).ifPresent(p::type);
                    });
                  });

              /** Create spec . */
              specs.add(new RouteSpecImpl(route, name, summary, desc, params, rsp));
            } else {
              specs.add((RouteSpec) candidate);
            }
          } else {
            log.debug("    ignoring {} {} from {}", route.method(), route.pattern(), owner);
          }
        } catch (Exception ex) {
          if (method != null) {
            // MVC:
            String name = method.getName();
            List<RouteParam> params = mvcParams(route, method, Collections.emptyMap());
            RouteResponseImpl rsp = new RouteResponseImpl(method.getGenericReturnType(), null,
                new HashMap<>());
            /** Create spec . */
            specs.add(new RouteSpecImpl(route, name, null, null, params, rsp));
          } else {
            if (cursor == null) {
              log.debug("ignoring {} {} no source code available", route.method(), route.pattern(),
                  ex);
              log.info("ignoring {} {} no source code available", route.method(), route.pattern());
            } else {
              log.error("ignoring {} {} reason {}", route.method(), route.pattern(), ex);
            }
          }
        }
      }
    } catch (Exception ex) {
      throw new IllegalStateException("Error while processing " + appClass, ex);
    }
    if (outdir != null) {
      save(outdir.resolve(appClass.getSimpleName() + ".spec"), specs);
    }
    log.debug("done");
    return specs;
  }

  private void save(final Path path, final List<RouteSpec> specs) {
    File fout = path.toFile();
    fout.getParentFile().mkdirs();
    try (ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(fout))) {
      log.info("    saving {}", fout);
      out.writeObject(specs);
    } catch (IOException ex) {
      throw Throwables.propagate(ex);
    }
  }

  private List<RouteParam> mvcParams(final Route.Definition route, final Method m,
      final Map<String, Object> doc) {
    RequestParamNameProviderImpl md = new RequestParamNameProviderImpl(
        new RouteMetadata(Env.DEFAULT.build(ConfigFactory.empty())));

    Parameter[] parameters = m.getParameters();
    List<RouteParam> params = new ArrayList<>(parameters.length);
    for (Parameter parameter : parameters) {

      if (parameter.isAnnotationPresent(Flash.class)
          || parameter.isAnnotationPresent(Local.class)) {
        continue;
      }
      if (SKIP.contains(parameter.getType())) {
        continue;
      }

      String name = md.name(parameter);

      final RouteParamType paramType;
      if (parameter.getAnnotation(Body.class) != null) {
        paramType = RouteParamType.BODY;
      } else if (parameter.getAnnotation(Header.class) != null) {
        paramType = RouteParamType.HEADER;
      } else if (route.vars().contains(name)) {
        paramType = RouteParamType.PATH;
      } else if (route.method().equals("GET")) {
        paramType = RouteParamType.QUERY;
      } else {
        paramType = RouteParamType.FORM;
      }

      String pdoc = (String) doc.get(name);

      RouteParamImpl param = new RouteParamImpl(name, parameter.getParameterizedType(),
          paramType, null, pdoc);
      params.add(param);
    }
    return params;
  }
}
