/*
 * Copyright 2017-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.mapping;

import org.onlab.util.Identifier;

/**
 * Representation of a Mapping ID.
 */
public final class MappingId extends Identifier<Long> {

    /**
     * Private constructor that prevents instantiation from external.
     *
     * @param id mapping identifier
     */
    private MappingId(long id) {
        super(id);
    }

    /**
     * Creates a mapping identifier from a long value.
     *
     * @param id mapping identifier in long format
     * @return mapping identifier
     */
    public static MappingId valueOf(long id) {
        return new MappingId(id);
    }

    /**
     * Obtains mapping identifier value.
     *
     * @return mapping identifier value as long
     */
    public long value() {
        return this.identifier;
    }

    @Override
    public String toString() {
        return Long.toHexString(identifier);
    }
}
